
#include "spi.h"
#include <util/delay.h>
#include <avr/io.h>

void begin() {

	// Set SS to high so a connected chip will be "deselected" by default
	PORTB |= _BV(SS);

	// When the SS pin is set as OUTPUT, it can be used as
	// a general purpose output port (it doesn't influence
	// SPI operations).
	DDRB |= _BV(SS);	

	// Warning: if the SS pin ever becomes a LOW INPUT then SPI
	// automatically switches to Slave, so the data direction of
	// the SS pin MUST be kept as OUTPUT.
	SPCR0 |= _BV(MSTR0);
	SPCR0 |= _BV(SPE0);

	// Set direction register for SCK and MOSI pin.
	// MISO pin automatically overrides to INPUT.
	// By doing this AFTER enabling SPI, we avoid accidentally
	// clocking in a single bit since the lines go directly
	// from "input" to SPI control.
	// http://code.google.com/p/arduino/issues/detail?id=888
	DDRB |= _BV(SCK);
	DDRB |= _BV(MOSI);
}


void end() {
	SPCR0 &= ~_BV(SPE0);
}

void setDataMode(uint8_t mode)
{
	SPCR0 = (SPCR0 & ~SPI_MODE_MASK) | mode;
}

void setClockDivider(uint8_t rate)
{
	SPCR0 = (SPCR0 & ~SPI_CLOCK_MASK) | (rate & SPI_CLOCK_MASK);
	SPSR0 = (SPSR0 & ~SPI_2XCLOCK_MASK) | ((rate >> 2) & SPI_2XCLOCK_MASK);
}
